#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Liste de fonction standard reutilisable

.. author: Dreamgeeker
"""

__author__ = "Ketsia LENTIN"
__copyright__ = "Copyright 2020, Couleur West IT"
__credits__ = "Ketsia LENTIN"
__license__ = "MIT"
__version__ = "1.2.5"
__email__ = "contact@couleurwest-it"
__status__ = "Production"
__docformat__ = 'reStructuredText'
